from django.apps import AppConfig


class ReimbursementsConfig(AppConfig):

    name = "reimbursements"

    def ready(self):
        # Import PyForms apps
        from .reimbursements_list import ReimbursementsApp
        from .reimbursement_form import RequestReimbursementForm

        #  and place them in the global scope
        global ReimbursementsApp
        global RequestReimbursementForm



from pyforms_web.widgets.django import ModelAdminWidget
from .reimbursement_form import RequestReimbursementForm
from pyforms_web.web.middleware import PyFormsMiddleware
from ..models import Reimbursement
from confapp import conf

class ReimbursementsApp(ModelAdminWidget):
    """
    """

    UID = "reimbursements"
    TITLE = "Reimbursements"

    MODEL = Reimbursement
    EDITFORM_CLASS = RequestReimbursementForm

    USE_DETAILS_TO_ADD  = False
    USE_DETAILS_TO_EDIT = False

    LIST_DISPLAY = [
        "created_by",
        "get_requisitions_status",
        "total",
        "status",
    ]
    LIST_FILTER = ["created", "status"]
    SEARCH_FIELDS = ["person__full_name__icontains", "ext_person_name__icontains"]

    # Orquestra ===============================================================
    LAYOUT_POSITION = conf.ORQUESTRA_HOME
    ORQUESTRA_MENU = "top"
    ORQUESTRA_MENU_ICON = "file alternate outline"
    ORQUESTRA_MENU_ORDER = 800
    # =========================================================================

    def __init__(self, *args, **kwargs):
        self.user = PyFormsMiddleware.user()
        super().__init__(*args, **kwargs)

        self._list.columns_size = ["60%", "10%", "10%", "10%", "10%"]

        self._list.columns_align = ["left"] * len(self.LIST_DISPLAY)
        self._list.columns_align[-2] = "right"
        self._list.columns_align[-1] = "center"




from django.core.exceptions import PermissionDenied

from reimbursements.models import Reimbursement


def print_reimbursement_form(request, reimbursement_id):
    reimbursement = Reimbursement.objects.get(pk=reimbursement_id)
    return reimbursement.generate_pdf_view(request)



# from django.conf import settings
# from django.contrib.auth import get_user_model
from confapp import conf
from django.urls import reverse
from pyforms.controls import ControlButton
from reimbursements.models import Reimbursement
from pyforms.basewidget import segment, no_columns
from pyforms_web.web.middleware import PyFormsMiddleware
from pyforms_web.widgets.django import ModelFormWidget
from .expense_list import ExpenseInline



class RequestReimbursementForm(ModelFormWidget):
    """
    """

    TITLE = "Request Reimbursement"

    MODEL = Reimbursement

    HAS_CANCEL_BTN_ON_ADD  = False
    HAS_CANCEL_BTN_ON_EDIT = False
    CLOSE_ON_REMOVE = True

    READ_ONLY = ["created", "modified", "status_changed", 'status']

    INLINES = [ExpenseInline]

    FIELDSETS = [
        no_columns('_previous', '_submit', "_print", '_printed', '_submit2approve','_reject', '_accept', 'bank_transfer_date', '_close', style="float:right"),
        "h3:Requester Information",
        segment(
            ("person", "iban")
        ),
        "h3:Expenses",
        segment("ExpenseInline"),
        ("created", "modified", "status", "status_changed")
    ]

    # Orquestra ===============================================================
    LAYOUT_POSITION = conf.ORQUESTRA_NEW_TAB
    # =========================================================================

    def __init__(self, *args, **kwargs):

        self._previous = ControlButton(
            '<i class="ui icon paper plane outline"></i>Previous status',
            css="basic gray",
            visible = False,
            label_visible=False,
            default = self.__previous_status_evt
        )

        self._submit = ControlButton(
            '<i class="ui icon paper plane outline"></i>Submit',
            css="blue",
            visible=False,
            label_visible=False,
            default = self.__submit_2_pending_evt
        )

        self._print = ControlButton(
            '<i class="ui icon print"></i>Print',
            visible=False,
            css="basic blue",
            label_visible=False
        )

        self._printed = ControlButton(
            '<i class="ui icon paper plane outline"></i>Set printed',
            visible=False,
            css="blue",
            label_visible=False,
            default=self.__set_printed_evt
        )

        self._submit2approve = ControlButton(
            '<i class="ui icon paper plane outline"></i>Submit to approval',
            visible=False,
            css="blue",
            label_visible=False,
            default=self.__submit_2_approve_evt
        )

        self._accept = ControlButton(
            '<i class="ui icon thumbs up"></i>Accept',
            visible=False,
            css="green",
            label_visible=False,
            default = self.__accept_evt
        )

        self._reject = ControlButton(
            '<i class="ui icon thumbs down"></i>Reject',
            visible=False,
            css="red",
            label_visible=False,
            default=self.__reject_evt
        )

        self._close = ControlButton(
            '<i class="ui icon thumbs up"></i>It was paid',
            visible=False,
            css="green basic",
            default=self.__set_closed_evt
        )

        super().__init__(*args, **kwargs)

        self.person.enabled = False

        if self.model_object is None:

            user   = PyFormsMiddleware.user()
            person = user.person_user.first()

            if person is not None:
                self.person.value = person.pk
                self.iban.value = person.privateinfo.iban
            else:
                self.warning(
                    'You need a Person profile associated to your account to be able to create reimbursements.'
                )

        self.update_fields_visibility()


    def update_fields_visibility(self):
        obj = self.model_object

        if obj:
            self._print.value = 'window.open("{0}", "_blank");'.format(
                reverse("print-reimbursement-form", args=[obj.pk])
            )
            self._print.show()
        else:
            self._print.hide()

        # show submit button only when is draft
        if obj is None or obj.status != 'draft':
            self._submit.hide()

        # show print button only when the reimbursement is pending
        if obj is None:
            self._print.hide()

        if obj and self.has_update_permissions():

            if obj.status == 'draft':

                self.bank_transfer_date.hide()
                self._submit.show()
                self._previous.hide()
                self._print.hide()
                self._printed.hide()
                self._submit2approve.hide()
                self._accept.hide()
                self._reject.hide()
                self._close.hide()
                self.iban.enabled = True

            elif obj.status == 'pending':

                self.bank_transfer_date.hide()
                self._previous.show()
                self._submit.hide()
                self._print.show()
                self._printed.show()
                self._submit2approve.hide()
                self._accept.hide()
                self._reject.hide()
                self._close.hide()
                self.iban.enabled = False

            elif obj.status == 'printed':

                self.bank_transfer_date.hide()
                self._previous.show()
                self._submit.hide()
                self._print.hide()
                self._printed.hide()
                self._submit2approve.show()
                self._accept.hide()
                self._reject.hide()
                self._close.hide()
                self.iban.enabled = False

            elif obj.status == 'submitted':

                self.bank_transfer_date.hide()
                self._previous.show()
                self._submit.hide()
                self._print.hide()
                self._printed.hide()
                self._submit2approve.hide()
                self._accept.show()
                self._reject.show()
                self._close.hide()
                self.iban.enabled = False

            elif obj.status in ['rejected', 'approved']:

                self._previous.hide()
                self._submit.hide()
                self._print.hide()
                self._printed.hide()
                self._submit2approve.hide()
                self._accept.hide()
                self._reject.hide()
                self.iban.enabled = False

                if obj.status == 'approved':
                    self.bank_transfer_date.show()
                    self._close.show()
                else:
                    self.bank_transfer_date.hide()
                    self._close.hide()

            elif obj.status == 'closed':

                self.bank_transfer_date.show()
                self.bank_transfer_date.readonly = True
                self._previous.hide()
                self._submit.hide()
                self._print.hide()
                self._printed.hide()
                self._submit2approve.hide()
                self._accept.hide()
                self._reject.hide()
                self._close.hide()
                self.iban.enabled = False

        else:

            self.bank_transfer_date.hide()
            self._submit.hide()
            self._previous.hide()
            self._print.hide()
            self._printed.hide()
            self._submit2approve.hide()
            self._accept.hide()
            self._reject.hide()
            self._close.hide()
            self.iban.enabled = True

        if obj.status == 'closed':
            self.bank_transfer_date.show()
            self.bank_transfer_date.readonly = True



    def save_btn_event(self):
        super().save_btn_event()
        self.update_fields_visibility()

    def __previous_status_evt(self):
        if not self.has_update_permissions():
            raise Exception('No permission')

        obj = self.model_object
        obj.previous_status()

        self.show_edit_form(self.object_pk)
        self.update_fields_visibility()

    def __set_printed_evt(self):
        if not self.has_update_permissions():
            raise Exception('No permission')

        obj = self.model_object
        obj.set_printed()

        self.show_edit_form(self.object_pk)
        self.update_fields_visibility()

    def __submit_2_pending_evt(self):
        """
        Notify the users with approval responsibility about the new reimbursement
        """
        if not self.has_update_permissions():
            raise Exception('No permission')

        obj = self.model_object
        obj.submit_to_pending()

        self.show_edit_form(self.object_pk)
        self.update_fields_visibility()

    def __submit_2_approve_evt(self):
        if not self.has_update_permissions():
            raise Exception('No permission')

        obj = self.model_object
        obj.submit_for_approval()

        self.show_edit_form(self.object_pk)
        self.update_fields_visibility()

    def __accept_evt(self):
        if not self.has_update_permissions():
            raise Exception('No permission')

        obj = self.model_object
        obj.accept()

        self.show_edit_form(self.object_pk)
        self.update_fields_visibility()

    def __reject_evt(self):
        if not self.has_update_permissions():
            raise Exception('No permission')

        obj = self.model_object
        obj.reject()

        self.show_edit_form(self.object_pk)
        self.update_fields_visibility()

    def __set_closed_evt(self):
        if not self.has_update_permissions():
            raise Exception('No permission')

        obj = self.model_object
        obj.bank_transfer_date = self.bank_transfer_date.value
        obj.set_closed() # this function will save the object

        self.show_edit_form(self.object_pk)
        self.update_fields_visibility()


    def get_readonly(self, default):
        res  = super().get_readonly(default)
        obj  = self.model_object

        if obj and obj.status in ['rejected', 'approved']:
            return res + ['status']

        if obj and obj.has_approve_permissions(PyFormsMiddleware.user()):
            return res
        else:
            return res + ['status']


    def update_object_fields(self, obj):
        """
        Update the created by field.
        :param Reimbursement obj:
        :return: Return the updated reimbursement object.
        """
        obj = super().update_object_fields(obj)

        if obj.pk is None:
            obj.created_by = PyFormsMiddleware.user()

        return obj

    @property
    def title(self):
        obj = self.model_object
        if obj:
            name = obj.requester_name
            total = obj.total
            return f"({obj.pk}) {obj.person} ({obj.total})"
        else:
            return super().title




from reimbursements.models import Expense
from pyforms_web.widgets.django import ModelAdminWidget
from .expense_form import ExpenseForm

class ExpenseInline(ModelAdminWidget):
    MODEL = Expense
    CLOSE_ON_REMOVE = True

    EDITFORM_CLASS = ExpenseForm

    LIST_DISPLAY = ['document_number',"requisition_number", 'expensecode', "short_description", "value"]
    LIST_HEADERS = ['Document number', 'Requisition number', 'Expense code','Short description', 'Amount']
    LIST_COLS_ALIGN = ["left", "center", "center", "left", "right"]
    LIST_COLS_SIZES = ["5%", "10%", '30%', "40%", "15%"]

    USE_DETAILS_TO_EDIT = False
    USE_DETAILS_TO_ADD =  False


    def has_add_permissions(self):
        obj = self.parent_model.objects.get(pk=self.parent_pk)
        return obj.status in ['draft', 'pending']

    def has_update_permissions(self, obj):
        obj = obj.reimbursement
        return obj.status in ['draft', 'pending']

    def has_remove_permissions(self, obj):
        obj = obj.reimbursement
        return obj.status in ['draft', 'pending']


import textwrap, os

from django.db import models
from djmoney.money import Money
from djmoney.models.fields import MoneyField
from django.core.exceptions import ValidationError
from djmoney.models.validators import MinMoneyValidator

def user_directory_path(instance, filename):
    return os.path.join('reimbursements', 'expense', instance.reimbursement.person.auth_user.username, filename)

class Expense(models.Model):
    """Expense to be reimbursed."""

    # TODO add attachments

    document_number    = models.CharField('Document number', max_length=50, blank=True)
    requisition_number = models.IntegerField('Requisition number', blank=True, null=True)
    description        = models.TextField("Description")
    value              = MoneyField(
                            "Amount",
                            max_digits=11,
                            decimal_places=2,
                            default_currency="EUR",
                            validators=[MinMoneyValidator(Money(0.01))],
                         )
    eur_value          = models.DecimalField(
                            'Amount in EUR',
                            max_digits=11,
                            decimal_places=2
                         )

    is_social     = models.BooleanField('It is a social expense', default=False)
    receipt       = models.FileField('Receipt', upload_to=user_directory_path)
    receipt_date  = models.DateField('Receipt date', null=True, default=None)
    reimbursement = models.ForeignKey(to="Reimbursement", on_delete=models.CASCADE, related_name="expenses")
    expensecode   = models.ForeignKey(to='finance.ExpenseCode', on_delete=models.CASCADE, related_name="reimbursement_expenses",
                                           verbose_name='Expense code')


    class Meta:
        ordering = ('document_number',)
        verbose_name = 'Reimbursement expense'
        verbose_name_plural = 'Reimbursements expenses'

    def clean(self):

        if self.description is not None:
            # restrict number of lines
            nlines = self.description.count("\n")
            if nlines > 8 or len(self.description) > 600:
                raise ValidationError({"description": "Too much text."})

    def short_description(self):
        return textwrap.shorten(self.description, width=100, placeholder="...")



from django.db import models
from django.db.models import Q
from permissions.models import Permission
from people.models import Group as ResearchGroup
from finance.models import ExpenseCode

class ReimbursementQuerySet(models.QuerySet):
    pass
    # User dependent Querysets
    # =========================================================================

    def owned_by(self, user):
        """
        Filters the Queryset to objects owned by the User or where
        he is referenced in a Foreign Key.

        This is by default what everyone sees if they have no permissions.
        """
        return self.filter( Q(created_by=user) | Q(person__auth_user=user) ).distinct()

    def managed_by(self, user, required_codenames, default=None):
        """
        Filters the Queryset to objects the user is allowed to manage
        given his Authorization Group profiles.

        Uses the RankedPermissions table.
        """
        if default is None:
            default = self.none()

        if user.is_superuser:
            return self

        ranked_permissions = Permission.objects.filter_by_user_and_auth_permissions(
            user, self.model, required_codenames)

        if ranked_permissions.exists():
            # check if the user has permissions to all registers
            if ranked_permissions.filter(researchgroup=None).exists():
                return self
            else:
                # check which expense codes the user has permissions to
                researchgroups = [p.researchgroup for p in ranked_permissions]
                auth_groups   = [g.groupdjango for g in researchgroups]
                expenses_with_access = ExpenseCode.objects.filter(
                    project__costcenter__group__in=auth_groups
                )

                # The owner of the reimbursements or the manager of the expense codes can access
                return self.filter(
                    Q(expenses__expensecode__in=expenses_with_access) |
                    Q(created_by=user) |
                    Q(person__auth_user=user)
                )

        return default.distinct()



    # PyForms Querysets

    def list_permissions(self, user):

        return self.managed_by(
            user,
            ['can_approve_reimbursements', 'manage_reimbursements', 'receive_approved_notifications'],
            default=self.owned_by(user)
        )

    def has_add_permissions(self, user):
        return True

    def has_view_permissions(self, user):
        # view_permission is useless because we let people see
        # their own objects via list_permissions
        return self.list_permissions(user)

    def has_update_permissions(self, user):

        # it was created or belongs to the user.
        if self.filter(
                Q(status="draft") &
                (
                    Q(created_by=user) | Q(person__auth_user=user)
                )
            ).exists():
            return True

        if self.filter(
                status__in=['draft', 'pending', 'printed', 'approved', 'rejected']
            ).managed_by( user, ['manage_reimbursements'] ).exists():
            return True

        if self.filter( status__in=['submitted'] ).managed_by( user, ['can_approve_reimbursements'] ).exists():
            return True

        return False

    def has_remove_permissions(self, user):
        """
        Only reimbursements in draft or pending can be removed.
        """
        return self.filter(
            Q(status__in=["draft", "pending"]) & ( Q(created_by=user) | Q(person__auth_user=user) )
        )




    # =========================================================================



from pyforms_web.web.middleware import PyFormsMiddleware
from reimbursements.models import Expense
from pyforms.controls import ControlAutoComplete
from finance.models import Project, CostCenter, ExpenseCode
from confapp import conf
from model_extra_fields.widgets.custom_model_form import CustomModelForm

class ExpenseForm(CustomModelForm):

    MODEL = Expense
    FIELDSETS = [
        ('document_number', "requisition_number", 'is_social'),
        ' ',
        ('_costcenter', '_financeprj', 'expensecode'),
        ('eur_value', "value", "value_currency"),
        ' ',
        ('receipt_date', ' ', ' '),
        'receipt',
        'select_form',
        'customized_form',
        ' ',
        'description'
    ]

    LAYOUT_POSITION = conf.ORQUESTRA_NEW_WINDOW

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.select_form.label = 'Extra information'

        self.value_currency.label = 'Amount currency'
        self.eur_value.changed_event = self.__update_value_evt

        if self.object_pk is None:
            self.document_number.value = self.parent.parent.model_object.expenses.count()+1
            self.expensecode.enabled = False
            self._financeprj.enabled = False


    def __update_value_evt(self):
        if self.eur_value.value and self.eur_value.value>0 and self.value.value==0:
            self.value.value = self.eur_value.value
            self.value_currency.value = 'EUR'


    def create_model_formfields(self):
        super().create_model_formfields()

        obj  = self.model_object
        user  = PyFormsMiddleware.user()
        person = user.person_user.first()

        self._costcenter = ControlAutoComplete(
            'Cost center',
            queryset=CostCenter.objects.active().by_person(person),
            changed_event=self.load_finance_projects,
            default=obj.expensecode.project.costcenter.pk if obj and obj.expensecode else None
        )
        self._financeprj = ControlAutoComplete(
            'Finance project',
            queryset=Project.objects.all(),
            enabled=False,
            changed_event=self.load_expense_codes,
            default=obj.expensecode.project.pk if obj and obj.expensecode else None
        )

        if obj and obj.expensecode:
            self.load_expense_codes()

        if person is None:
            self.warning('Sorry you require a person associated to your user.')
            return



    def load_finance_projects(self):
        self._financeprj.queryset = Project.objects.filter(costcenter=self._costcenter.value)
        self._financeprj.value = None
        self._financeprj.enabled = True
        self.expensecode.enabled = False

    def load_expense_codes(self):
        self.expensecode.queryset = ExpenseCode.objects.filter(project=self._financeprj.value)
        self.expensecode.value = None
        self.expensecode.enabled = True





from permissions.models import Permission
from django.db import models
from model_utils import Choices
from django.conf import settings
from django.shortcuts import render
from django.utils.html import format_html
from django_weasyprint import WeasyTemplateView
from localflavor.generic.models import IBANField
from django.core.exceptions import ValidationError
from model_utils.models import StatusModel, TimeStampedModel
from .reimbursement_queryset import ReimbursementQuerySet
from notifications.tools import notify

class Reimbursement(StatusModel, TimeStampedModel):

    STATUS = Choices(
        ("draft",     "Draft"),
        ("pending",   "Pending"),
        ("printed",   "Printed"),
        ("submitted", "Submitted"),
        ("approved",  "Approved"),
        ("rejected",  "Rejected"),
        ("closed",    "Closed"),
    )

    created_by = models.ForeignKey(to=settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    person = models.ForeignKey(to="people.Person", on_delete=models.CASCADE)
    iban = IBANField(verbose_name="IBAN", blank=True, include_countries=("PT",))
    order = models.ForeignKey('orders.Order', on_delete=models.CASCADE, null=True, blank=True)

    bank_transfer_date = models.DateField('Transfer date', null=True, blank=True)

    objects = ReimbursementQuerySet.as_manager()

    class Meta:
        ordering = ("-created",)
        permissions = (
            ("manage_reimbursements", "Submit reimbursements for approval"),
            ("can_approve_reimbursements", "Approve or reject reimbursements"),
            ("receive_approved_notifications", "Receive notifications about the approved reimbursements"),
        )
        verbose_name = 'Reimbursement'
        verbose_name_plural = 'Reimbursements'

    def __str__(self):
        return f"Reimbursement for {self.person}"

    def clean(self):
        if self.created_by is None:
            raise ValidationError("This proposal has no person responsible")

    def has_approve_permissions(self, user):
        if self.pk is None or self.status == 'draft':
            return False

        perms = Permission.objects.filter_by_auth_permissions(
            Reimbursement, ['can_approve_reimbursements']
        )

        for p in perms:
            if p.researchgroup is None and user in p.auth_group.user_set.all():
                return True

        # get all django groups associated to the expenses
        exp_auth_groups = [e.expensecode.project.costcenter.group for e in self.expenses.all()]
        # get all the research groups associated to the expenses
        exp_researchgrps = [g.group_django.first() for g in exp_auth_groups if
                            g.group_django.first() is not None]

        # check if the user has permissions to all the expense codes research groups
        perms = perms.filter(researchgroup__in=exp_researchgrps).distinct()
        has_access = perms.count() > 0
        for p in perms:
            if user not in p.auth_group.user_set.all():
                has_access = False
                break

        return has_access

    @property
    def requester_name(self):
        return self.person.full_name if self.person else 'Undefined person'

    @property
    def requester_iban(self):
        return "".join(d + " " if (i + 1) % 4 == 0 else d for i, d in enumerate(self.iban))

    @property
    def total(self):
        try:
            return sum(expense.value for expense in self.expenses.all())
        except TypeError:
            # can not add Money with different currencies
            return None

    def get_requisitions_status(self):
        total_expenses = self.expenses.count()
        with_req_num = self.expenses.filter(requisition_number__isnull=False).count()

        if total_expenses > 0 and total_expenses == with_req_num:
            icon = '<i class="green check icon"></i>'
        else:
            icon = '<i class="red times icon"></i>'

        return f"{icon} ({with_req_num}/{total_expenses})"

    get_requisitions_status.short_description = "requisitions"

    def status_icon(self):
        d = {
            "draft": '<i class="grey edit outline icon"></i>',
            "pending": '<i class="grey clock icon"></i>',
            "printed": '<i class="blue print icon"></i>',
            "submitted": '<i class="black lock icon"></i>',
            "approved": '<i class="green thumbs up icon"></i>',
            "rejected": '<i class="red thumbs down icon"></i>',
        }

        icon = d.get(self.status, "")
        return format_html(f'<div align="center">{icon}</div>')

    status_icon.short_description = "status"

    def generate_pdf_view(self, request=None):
        """Returns a PDF view for this proposal."""

        template = "reimbursements/reimbursement_print.html"

        context = {
            "reimbursement": self,
            # # FIXME test only
            # 'motive': 'New Hire',
            # 'proposal_date': self.contractproposal_createdon.strftime('%b %d, %Y'),
        }

        # FIXME this should be the debug, erase the `not`
        if "debug" not in request.GET:
            return render(request, template_name=template, context=context)

        return WeasyTemplateView.as_view(template_name=template, extra_context=context)(
            request
        )

    def get_users_with_permissions(self, permission_code):
        perms = Permission.objects.filter_by_auth_permissions(
            Reimbursement, [permission_code]
        )

        exp_auth_groups = [e.expensecode.project.costcenter.group for e in self.expenses.all()]
        exp_researchgrps = [g.group_django.first() for g in exp_auth_groups if g.group_django.first() is not None]
        sent_to_groups = [p.auth_group for p in perms if
                          p.researchgroup is None or p.researchgroup in exp_researchgrps]
        users = set()
        for g in sent_to_groups:
            for u in g.user_set.all():
                users.add(u)

        return users

    # =========================================================================
    # FUNCTIONS TO CONTROL THE STATUS #########################################
    # =========================================================================

    def previous_status(self):

        if self.status=='pending':
            self.status='draft'

        elif self.status=='printed':
            self.status='pending'

        elif self.status=='submitted':
            self.status='printed'

        self.save()



    def submit_to_pending(self):
        if self.expenses.count() <= 0:
            raise Exception("Add at least one Expense to be reimbursed.")

        self.status = 'pending'
        self.save()

        for user in self.get_users_with_permissions('can_approve_reimbursements'):

            notify(
                'NEW_REIMBURSEMENT_REQUEST',
                '{} submitted a new reimbursement request'.format(self.person),
                '{} submitted a new reimbursement request'.format(self.person),
                user,
                visible=True,
                label='New reimbursement request',
                period='D'
            )


    def set_printed(self):
        self.status = 'printed'
        self.save()

    def submit_for_approval(self):
        self.status = 'submitted'
        self.save()

        for user in self.get_users_with_permissions('can_approve_reimbursements'):

            notify(
                'REIMBURSEMENT_APPROVAL_REQUEST',
                'The reimbursement from the user {} requires our approval'.format(self.person),
                'The reimbursement from the user {} requires our approval'.format(self.person),
                user,
                visible=True,
                label='Reimbursement approval request',
                period='D'
            )


    def accept(self):
        self.status = 'approved'
        self.save()

        # Notify the users that a new reimbursement was approved.
        for user in self.get_users_with_permissions('receive_approved_notifications'):
            notify(
                'ONE_REIMBURSEMENT_WAS_APPROVED',
                'A new reimbursement for the user {} was approved'.format(self.person),
                'A new reimbursement for the user {} was approved'.format(self.person),
                user,
                visible=True,
                label='Approved reimbursement notification',
                period='D'
            )

        notify(
            'YOUR_REIMBURSEMENT_WAS_APPROVED',
            'Your reimbursement was approved',
            'Your reimbursement was approved',
            self.person.auth_user,
            visible=True,
            label='Notification about your reimbursement approval',
            period='D'
        )

    def reject(self):
        self.status = 'rejected'
        self.save()

        notify(
            'YOUR_REIMBURSEMENT_WAS_REJECTED',
            'Your reimbursement was rejected',
            'Your reimbursement was rejected',
            self.person.auth_user,
            visible=True,
            label='Notification about your reimbursement rejection',
            period='I'
        )

    def set_closed(self):
        self.status = 'closed'
        self.save()


